/*!
 *  \file    test_lcd.c
 *  \author  Wim Dolman (<a href="email:w.e.dolman@hva.nl">w.e.dolman@hva.nl</a>)
 *  \date    21-10-2013
 *  \version 1.0
 *
 *  \brief   test file lcd driver for Xmega devices
 *
 *  \details This file contains the test for lcd library.
 *           This testprogram can be used for all four modes
 *           (4-bit/8-bit,busyfalg/no busyflag)
 *           It uses the default settings in lcd.h
 *           The data lines are connected to port A.
 *           The control lines R/W, RS and E are connect to
 *           the pins 3, 4 and 5 of port D.
 *           The circuit is in schematic below:
 *  \image   html  xmega_schematic_test_lcd.png
 *  \image   latex xmega_schematic_test_lcd.eps
 *           The default mode is 4-bit with no busyflag.
 *           You can changes the mode with the macro's LCD_4BIT_MODE and
 *           LCD_BUSY_FLAG in the lcd.h, but in that case you need to recompile
 *           both lcd.c and test_lcd.c.
 *
 *  \warning Be careful using the busyflag. Most alfanumeric displays are 5 Volt devices.
 *           Xmega's are not 5 Volt tolerant.
 *           When using the busyflag, information is also send from the LCD to the
 *           Xmega. So you can damage your Xmega.
 *           In this schematic is a 3.3 V LCD used.
 *           Add a levelshifter to the data lines if you want to use a 5 V LCD with
 *           a busy flag, e.g. the TXB0108 from Texas Instruments.
 *
 *           The schematic below gives an example with a 5 V display without using
 *           the busy flag.
 *           Input R/W off the display is now connected to the reference.
 *  \image   html  xmega_schematic_test2_lcd.png
 *  \image   latex xmega_schematic_test2_lcd.eps
 *           The data lines in lcd.h has to be changed and the 4-bit mode
 *           and no busy flag must be selected:
 * \verbatim
      #define LCD_4BIT_MODE     1
      #define LCD_BUSY_FLAG     0
      #define LCD_DATA_PORT     PORTD
      #define LCD_D4_bp         PIN0_bp
      #define LCD_D5_bp         PIN1_bp
      #define LCD_D6_bp         PIN2_bp
      #define LCD_D7_bp         PIN3_bp
      #define LCD_COMM_PORT     PORTD
      #define LCD_RS_PORT       LCD_COMM_PORT
      #define LCD_E_PORT        LCD_COMM_PORT
      #define LCD_RS_bp         PIN4_bp
      #define LCD_E_bp          PIN5_bp
   \endverbatim
 *           After changing lcd.h both lcd.c en test_lcd.c must be recompiled.
 */

/*!
 *  \brief F_CPU must be declared
*/
#ifndef F_CPU
#define F_CPU             2000000UL
#endif

#include <avr/io.h>
#include <util/delay.h>
#include <stdlib.h>
#include "lcd.h"

/*! \brief Defines unused data lines
 *
 *  This function makes unused data lines output and makes
 *  this outptus low.
 *
 *  \return           none
 */
void define_unused_data_lines(void)
{
  if ( LCD_4BIT_MODE ) {
    LCD_DATA_PORT.DIRSET = ((uint8_t) ~(LCD_DATA_PORT_gm));
    LCD_DATA_PORT.OUTCLR = ((uint8_t) ~(LCD_DATA_PORT_gm));
  }
}

/*! \brief Defines unused R/W line
 *
 *  This function makes unused r/w line output and make it low.
 *
 *  \return           none
 */
void define_unused_rw_line(void)
{
  if ( ! LCD_BUSY_FLAG  ) {
    LCD_RW_PORT.DIRSET = LCD_RW_bm;
    LCD_RW_PORT.OUTCLR = LCD_RW_bm;
  }
}

/*! \brief Print LCD mode
 *
 *  This function printf 4 bit if the 4-bit mode is used and
 *  8 bit if the 8-bit mode is used
 *
 *  \return           none
 */
void lcd_print_mode(void)
{
  if ( LCD_4BIT_MODE ) {
    lcd_puts("4 bit");
  } else {
    lcd_puts("8 bit");
  }
}

/*! \brief Print if busyflag is used
 *
 *  This function printfs BF is busy flag is used
 *  and NO BF if no busyflag is used
 *
 *  \return           none
 */
void lcd_print_busy_flag(void)
{
  if (! LCD_BUSY_FLAG ) {
    lcd_puts("NO ");
  }
  lcd_puts("BF");
}

/*! \brief Main testprogram
 *
 *  This program prints on the first line of the LCD
 *  the text "Test" and a number counting from 0 until 999.
 *  On the second line it prints the LCD mode and
 *  if it uses the busy flag or not.
 *
 *  \return           none
 */
int main(void)
{
  uint16_t i = 0;
  char buffer[8];

  define_unused_data_lines();
  define_unused_rw_line();

  lcd_init();

  while (1) {
    lcd_clear();

    // Prints "Test"
    lcd_puts("Test");

    // Prints i
    lcd_gotoxy(8,0);
    utoa(i, buffer, 10);
    lcd_puts(buffer);

    // Prints LCD mode
    lcd_gotoxy(0,1);
    lcd_print_mode();

    // Prints busy flag
    lcd_gotoxy(8,1);
    lcd_print_busy_flag();

    _delay_ms(500);

    if (++i == 1000) {
      i=0;
    }
  }
}